@@ -17,13 +17,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-from __future__ import print_function
 from operator import itemgetter
-from PyQt4 import QtCore
+from PyQt5 import QtCore
 from picard import (PICARD_APP_NAME, PICARD_ORG_NAME, PICARD_VERSION,
                     version_to_string, version_from_string)
 from picard.util import LockableObject
-
+from picard import log
 
 class ConfigUpgradeError(Exception):
     pass
@@ -34,9 +33,27 @@ class ConfigSection(LockableObject):
     """Configuration section."""
 
     def __init__(self, config, name):
-        LockableObject.__init__(self)
-        self.__config = config
+        super().__init__()
+        self.__qt_config = config
+        self.__config = {}
         self.__name = name
+        self.__load_keys()
+
+    def __qt_keys(self):
+        prefix = self.__name + '/'
+        return filter(lambda key: key.startswith(prefix),
+                      self.__qt_config.allKeys())
+
+    def __load_keys(self):
+        for key in self.__qt_keys():
+            try:
+                self.__config[key] = self.__qt_config.value(key)
+            except TypeError:
+                # Related to PICARD-1255, Unable to load the object into
+                # Python at all. Something weird with the way it is read and converted
+                # via the Qt C++ API. Simply ignore the key and it will be reset to
+                # default whenever the user opens Picard options
+                log.error('Unable to load config value: %s', key)
 
     def __getitem__(self, name):
         opt = Option.get(self.__name, name)
@@ -45,42 +62,42 @@ class ConfigSection(LockableObject):
         return self.value(name, opt, opt.default)
 
     def __setitem__(self, name, value):
+        key = self.__name + '/' + name
         self.lock_for_write()
         try:
-            self.__config.setValue("%s/%s" % (self.__name, name), value)
+            self.__config[key] = value
+            self.__qt_config.setValue(key, value)
         finally:
             self.unlock()
 
-    def __contains__(self, key):
-        key = "%s/%s" % (self.__name, key)
-        return self.__config.contains(key)
+    def __contains__(self, name):
+        key = self.__name + '/' + name
+        return key in self.__config
 
-    def remove(self, key):
-        key = "%s/%s" % (self.__name, key)
-        if self.__config.contains(key):
-            self.__config.remove(key)
+    def remove(self, name):
+        key = self.__name + '/' + name
+        self.lock_for_write()
+        try:
+            if key in self.__config:
+                self.__config.pop(key)
+                self.__qt_config.remove(key)
+        finally:
+            self.unlock()
 
-    def raw_value(self, key):
+    def raw_value(self, name):
         """Return an option value without any type conversion."""
-        value = self.__config.value("%s/%s" % (self.__name, key))
-
-        # XXX QPyNullVariant does not exist in all PyQt versions, and was
-        # removed entirely in PyQt5. See:
-        # http://pyqt.sourceforge.net/Docs/PyQt5/pyqt_qvariant.html
-        if str(type(value)) == "<class 'PyQt4.QtCore.QPyNullVariant'>":
-            return ""
-
+        value = self.__config[self.__name + '/' + name]
         return value
 
-    def value(self, name, type, default=None):
+    def value(self, name, option_type, default=None):
         """Return an option value converted to the given Option type."""
-        key = "%s/%s" % (self.__name, name)
+        key = self.__name + '/' + name
         self.lock_for_read()
         try:
-            if self.__config.contains(key):
-                return type.convert(self.raw_value(name))
+            if key in self.__config:
+                return option_type.convert(self.raw_value(name))
             return default
-        except:
+        except Exception:
             return default
         finally:
             self.unlock()
@@ -138,7 +155,7 @@ class Config(QtCore.QSettings):
 
     def switchProfile(self, profilename):
         """Sets the current profile."""
-        key = u"profile/%s" % (profilename,)
+        key = "profile/%s" % (profilename,)
         if self.contains(key):
             self.profile.name = key
         else:
@@ -211,6 +228,7 @@ class Option(QtCore.QObject):
     registry = {}
 
     def __init__(self, section, name, default):
+        super().__init__()
         self.section = section
         self.name = name
         self.default = default
@@ -225,7 +243,7 @@ class Option(QtCore.QObject):
 
 class TextOption(Option):
 
-    convert = unicode
+    convert = str
 
 
 class BoolOption(Option):
@@ -258,8 +276,8 @@ class ListOption(Option):
 class IntListOption(Option):
 
     @staticmethod
-    def convert(value):
-        return map(int, value)
+    def convert(values):
+        return list(map(int, values))
 
 
 config = None
