@@ -22,10 +22,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-import json
-import traceback
-from PyQt4 import QtCore, QtGui
-from PyQt4.QtNetwork import QNetworkReply
+from collections import OrderedDict, namedtuple
+from PyQt5 import QtCore, QtWidgets
+from PyQt5.QtNetwork import QNetworkReply
 from picard import config, log
 from picard.const import CAA_HOST, CAA_PORT
 from picard.coverart.providers import CoverArtProvider, ProviderOptions
@@ -33,27 +32,56 @@ from picard.coverart.image import CaaCoverArtImage, CaaThumbnailCoverArtImage
 from picard.coverart.utils import CAA_TYPES, translate_caa_type
 from picard.ui.ui_provider_options_caa import Ui_CaaOptions
 from picard.ui.util import StandardButton
-from picard.util import webbrowser2
-from picard.ui.options import OptionsPage, OptionsCheckError, register_options_page
+from picard.util import webbrowser2, load_json
 
 
-_CAA_THUMBNAIL_SIZE_MAP = {
-    0: "small",
-    1: "large",
-}
+CaaSizeItem = namedtuple('CaaSizeItem', ['thumbnail', 'label'])
 
-class CAATypesSelectorDialog(QtGui.QDialog):
+_CAA_THUMBNAIL_SIZE_MAP = OrderedDict([
+    (250, CaaSizeItem('small', N_('250 px'))),
+    (500, CaaSizeItem('large', N_('500 px'))),
+    (1200, CaaSizeItem('1200', N_('1200 px'))),
+    (-1, CaaSizeItem(None, N_('Full size'))),
+])
+
+_CAA_IMAGE_SIZE_DEFAULT = 500
+
+
+def caa_url_fallback_list(desired_size, thumbnails):
+    """List of thumbnail urls equal or smaller than size, in size decreasing order
+    It is used for find the "best" thumbnail according to:
+        - user choice
+        - thumbnail availability
+    If user choice isn't matching an available thumbnail size, a fallback to
+    smaller thumbnails is possible
+    This function returns the list of possible urls, ordered from the biggest
+    matching the user choice to the smallest one.
+    Of course, if none are possible, the returned list may be empty.
+    """
+    reversed_map = OrderedDict(reversed(list(_CAA_THUMBNAIL_SIZE_MAP.items())))
+    urls = []
+    for item_id, item in reversed_map.items():
+        if item_id == -1 or item_id > desired_size:
+            continue
+        if item.thumbnail in thumbnails:
+            urls.append(thumbnails[item.thumbnail])
+    return urls
+
+
+class CAATypesSelectorDialog(QtWidgets.QDialog):
     _columns = 4
 
-    def __init__(self, parent=None, types=[]):
-        super(CAATypesSelectorDialog, self).__init__(parent)
+    def __init__(self, parent=None, types=None):
+        if types is None:
+            types = []
+        super().__init__(parent)
 
         self.setWindowTitle(_("Cover art types"))
         self._items = {}
-        self.layout = QtGui.QVBoxLayout(self)
+        self.layout = QtWidgets.QVBoxLayout(self)
 
-        grid = QtGui.QWidget()
-        gridlayout = QtGui.QGridLayout()
+        grid = QtWidgets.QWidget()
+        gridlayout = QtWidgets.QGridLayout()
         grid.setLayout(gridlayout)
 
         for index, caa_type in enumerate(CAA_TYPES):
@@ -61,40 +89,38 @@ class CAATypesSelectorDialog(QtGui.QDialog):
             column = index % self._columns
             name = caa_type["name"]
             text = translate_caa_type(name)
-            item = QtGui.QCheckBox(text)
+            item = QtWidgets.QCheckBox(text)
             item.setChecked(name in types)
             self._items[item] = caa_type
             gridlayout.addWidget(item, row, column)
 
         self.layout.addWidget(grid)
 
-        self.buttonbox = QtGui.QDialogButtonBox(self)
+        self.buttonbox = QtWidgets.QDialogButtonBox(self)
         self.buttonbox.setOrientation(QtCore.Qt.Horizontal)
         self.buttonbox.addButton(
-            StandardButton(StandardButton.OK), QtGui.QDialogButtonBox.AcceptRole)
+            StandardButton(StandardButton.OK), QtWidgets.QDialogButtonBox.AcceptRole)
         self.buttonbox.addButton(StandardButton(StandardButton.CANCEL),
-                                 QtGui.QDialogButtonBox.RejectRole)
+                                 QtWidgets.QDialogButtonBox.RejectRole)
         self.buttonbox.addButton(
-            StandardButton(StandardButton.HELP), QtGui.QDialogButtonBox.HelpRole)
-        self.buttonbox.accepted.connect(self.accept)
-        self.buttonbox.rejected.connect(self.reject)
-        self.buttonbox.helpRequested.connect(self.help)
+            StandardButton(StandardButton.HELP), QtWidgets.QDialogButtonBox.HelpRole)
 
         extrabuttons = [
             (N_("Chec&k all"), self.checkall),
             (N_("&Uncheck all"), self.uncheckall),
         ]
         for label, callback in extrabuttons:
-            button = QtGui.QPushButton(_(label))
+            button = QtWidgets.QPushButton(_(label))
             button.setSizePolicy(
-                QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Expanding)
-            self.buttonbox.addButton(button, QtGui.QDialogButtonBox.ActionRole)
+                QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
+            self.buttonbox.addButton(button, QtWidgets.QDialogButtonBox.ActionRole)
             button.clicked.connect(callback)
 
         self.layout.addWidget(self.buttonbox)
 
         self.buttonbox.accepted.connect(self.accept)
         self.buttonbox.rejected.connect(self.reject)
+        self.buttonbox.helpRequested.connect(self.help)
 
     def help(self):
         webbrowser2.goto('doc_cover_art_types')
@@ -110,19 +136,16 @@ class CAATypesSelectorDialog(QtGui.QDialog):
             item.setChecked(value)
 
     def get_selected_types(self):
-        types = []
-        for item, typ in self._items.iteritems():
-            if item.isChecked():
-                types.append(typ['name'])
-        if not types:
-            return [u'front']
-        return types
+        return [typ['name'] for item, typ in self._items.items() if
+                item.isChecked()] or ['front']
 
     @staticmethod
-    def run(parent=None, types=[]):
+    def run(parent=None, types=None):
+        if types is None:
+                types = []
         dialog = CAATypesSelectorDialog(parent, types)
         result = dialog.exec_()
-        return (dialog.get_selected_types(), result == QtGui.QDialog.Accepted)
+        return (dialog.get_selected_types(), result == QtWidgets.QDialog.Accepted)
 
 
 class ProviderOptionsCaa(ProviderOptions):
@@ -134,30 +157,45 @@ class ProviderOptionsCaa(ProviderOptions):
         config.BoolOption("setting", "caa_save_single_front_image", False),
         config.BoolOption("setting", "caa_approved_only", False),
         config.BoolOption("setting", "caa_image_type_as_filename", False),
-        config.IntOption("setting", "caa_image_size", 1),
-        config.ListOption("setting", "caa_image_types", [u"front"]),
+        config.IntOption("setting", "caa_image_size",
+                         _CAA_IMAGE_SIZE_DEFAULT),
+        config.ListOption("setting", "caa_image_types", ["front"]),
         config.BoolOption("setting", "caa_restrict_image_types", True),
     ]
 
     _options_ui = Ui_CaaOptions
 
     def __init__(self, parent=None):
-        super(ProviderOptionsCaa, self).__init__(parent)
+        super().__init__(parent)
         self.ui.restrict_images_types.clicked.connect(self.update_caa_types)
         self.ui.select_caa_types.clicked.connect(self.select_caa_types)
 
+    def restore_defaults(self):
+        self.caa_image_types = ["front"]
+        super().restore_defaults()
+
     def load(self):
-        self.ui.cb_image_size.setCurrentIndex(config.setting["caa_image_size"])
+        self.ui.cb_image_size.clear()
+        for item_id, item in _CAA_THUMBNAIL_SIZE_MAP.items():
+            self.ui.cb_image_size.addItem(_(item.label), userData=item_id)
+
+        size = config.setting["caa_image_size"]
+        index = self.ui.cb_image_size.findData(size)
+        if index < 0:
+            index = self.ui.cb_image_size.findData(_CAA_IMAGE_SIZE_DEFAULT)
+        self.ui.cb_image_size.setCurrentIndex(index)
+
         self.ui.cb_save_single_front_image.setChecked(config.setting["caa_save_single_front_image"])
         self.ui.cb_approved_only.setChecked(config.setting["caa_approved_only"])
         self.ui.cb_type_as_filename.setChecked(config.setting["caa_image_type_as_filename"])
         self.ui.restrict_images_types.setChecked(
             config.setting["caa_restrict_image_types"])
+        self.caa_image_types = config.setting["caa_image_types"]
         self.update_caa_types()
 
     def save(self):
-        config.setting["caa_image_size"] = \
-            self.ui.cb_image_size.currentIndex()
+        size = self.ui.cb_image_size.currentData()
+        config.setting["caa_image_size"] = size
         config.setting["caa_save_single_front_image"] = \
             self.ui.cb_save_single_front_image.isChecked()
         config.setting["caa_approved_only"] = \
@@ -166,6 +204,7 @@ class ProviderOptionsCaa(ProviderOptions):
             self.ui.cb_type_as_filename.isChecked()
         config.setting["caa_restrict_image_types"] = \
             self.ui.restrict_images_types.isChecked()
+        config.setting["caa_image_types"] = self.caa_image_types
 
     def update_caa_types(self):
         enabled = self.ui.restrict_images_types.isChecked()
@@ -173,9 +212,9 @@ class ProviderOptionsCaa(ProviderOptions):
 
     def select_caa_types(self):
         (types, ok) = CAATypesSelectorDialog.run(
-            self, config.setting["caa_image_types"])
+            self, self.caa_image_types)
         if ok:
-            config.setting["caa_image_types"] = types
+            self.caa_image_types = types
 
 
 
@@ -184,7 +223,7 @@ class CoverArtProviderCaa(CoverArtProvider):
     """Get cover art from Cover Art Archive using release mbid"""
 
     NAME = "Cover Art Archive"
-    TITLE = N_(u'Cover Art Archive')
+    TITLE = N_('Cover Art Archive')
     OPTIONS = ProviderOptionsCaa
 
     ignore_json_not_found_error = False
@@ -192,8 +231,8 @@ class CoverArtProviderCaa(CoverArtProvider):
     coverartimage_thumbnail_class = CaaThumbnailCoverArtImage
 
     def __init__(self, coverart):
-        CoverArtProvider.__init__(self, coverart)
-        self.caa_types = map(unicode.lower, config.setting["caa_image_types"])
+        super().__init__(coverart)
+        self.caa_types = list(map(str.lower, config.setting["caa_image_types"]))
         self.len_caa_types = len(self.caa_types)
         self.restrict_types = config.setting["caa_restrict_image_types"]
 
@@ -201,24 +240,24 @@ class CoverArtProviderCaa(CoverArtProvider):
     def _has_suitable_artwork(self):
         # MB web service indicates if CAA has artwork
         # https://tickets.metabrainz.org/browse/MBS-4536
-        if 'cover_art_archive' not in self.release.children:
+        if 'cover-art-archive' not in self.release:
             log.debug("No Cover Art Archive information for %s"
-                      % self.release.id)
+                      % self.release['id'])
             return False
 
-        caa_node = self.release.children['cover_art_archive'][0]
-        caa_has_suitable_artwork = caa_node.artwork[0].text == 'true'
+        caa_node = self.release['cover-art-archive']
+        caa_has_suitable_artwork = caa_node['artwork']
 
         if not caa_has_suitable_artwork:
             log.debug("There are no images in the Cover Art Archive for %s"
-                      % self.release.id)
+                      % self.release['id'])
             return False
 
         if self.restrict_types:
             want_front = 'front' in self.caa_types
             want_back = 'back' in self.caa_types
-            caa_has_front = caa_node.front[0].text == 'true'
-            caa_has_back = caa_node.back[0].text == 'true'
+            caa_has_front = caa_node['front']
+            caa_has_back = caa_node['back']
 
             if self.len_caa_types == 2 and (want_front or want_back):
                 # The OR cases are there to still download and process the CAA
@@ -241,16 +280,16 @@ class CoverArtProviderCaa(CoverArtProvider):
 
         if not caa_has_suitable_artwork:
             log.debug("There are no suitable images in the Cover Art Archive for %s"
-                      % self.release.id)
+                      % self.release['id'])
         else:
             log.debug("There are suitable images in the Cover Art Archive for %s"
-                      % self.release.id)
+                      % self.release['id'])
 
         return caa_has_suitable_artwork
 
     def enabled(self):
         """Check if CAA artwork has to be downloaded"""
-        if not super(CoverArtProviderCaa, self).enabled() or \
+        if not super().enabled() or \
                 self.coverart.front_image_found:
             return False
         if self.restrict_types and not self.len_caa_types:
@@ -263,7 +302,7 @@ class CoverArtProviderCaa(CoverArtProvider):
         return "/release/%s/" % self.metadata["musicbrainz_albumid"]
 
     def queue_images(self):
-        self.album.tagger.xmlws.download(
+        self.album.tagger.webservice.download(
             CAA_HOST,
             CAA_PORT,
             self._caa_path,
@@ -280,15 +319,13 @@ class CoverArtProviderCaa(CoverArtProvider):
         self.album._requests -= 1
         if error:
             if not (error == QNetworkReply.ContentNotFoundError and self.ignore_json_not_found_error):
-                self.error(u'CAA JSON error: %s' % (unicode(http.errorString())))
+                self.error('CAA JSON error: %s' % (http.errorString()))
         else:
             try:
-                caa_data = json.loads(data)
+                caa_data = load_json(data)
             except ValueError:
                 self.error("Invalid JSON: %s" % (http.url().toString()))
             else:
-                imagesize = config.setting["caa_image_size"]
-                thumbsize = _CAA_THUMBNAIL_SIZE_MAP.get(imagesize, None)
                 for image in caa_data["images"]:
                     if config.setting["caa_approved_only"] and not image["approved"]:
                         continue
@@ -300,9 +337,9 @@ class CoverArtProviderCaa(CoverArtProvider):
                     # if image has no type set, we still want it to match
                     # pseudo type 'unknown'
                     if not image["types"]:
-                        image["types"] = [u"unknown"]
+                        image["types"] = ["unknown"]
                     else:
-                        image["types"] = map(unicode.lower, image["types"])
+                        image["types"] = list(map(str.lower, image["types"]))
                     if self.restrict_types:
                         # only keep enabled caa types
                         types = set(image["types"]).intersection(
@@ -310,21 +347,24 @@ class CoverArtProviderCaa(CoverArtProvider):
                     else:
                         types = True
                     if types:
-                        if thumbsize is None or is_pdf:
+                        urls = caa_url_fallback_list(config.setting["caa_image_size"],
+                                                     image["thumbnails"])
+                        if not urls or is_pdf:
                             url = image["image"]
                         else:
-                            url = image["thumbnails"][thumbsize]
+                            #FIXME: try other urls in case of 404
+                            url = urls[0]
                         coverartimage = self.coverartimage_class(
                             url,
                             types=image["types"],
                             is_front=image['front'],
                             comment=image["comment"],
                         )
-                        if is_pdf:
+                        if urls and is_pdf:
                             # thumbnail will be used to "display" PDF in info
                             # dialog
                             thumbnail = self.coverartimage_thumbnail_class(
-                                url=image["thumbnails"]['small'],
+                                url=url[0],
                                 types=image["types"],
                                 is_front=image['front'],
                                 comment=image["comment"],
