@@ -20,24 +20,25 @@
 import os
 import re
 from functools import partial
-from PyQt4 import QtCore, QtGui
+from PyQt5 import QtCore, QtGui, QtWidgets
 from picard import config, log
 from picard.album import Album, NatAlbum
-from picard.cluster import Cluster, ClusterList, UnmatchedFiles
+from picard.cluster import Cluster, ClusterList, UnclusteredFiles
 from picard.file import File
 from picard.track import Track, NonAlbumTrack
-from picard.util import encode_filename, icontheme
+from picard.util import encode_filename, icontheme, restore_method
 from picard.plugin import ExtensionPoint
 from picard.ui.ratingwidget import RatingWidget
 from picard.ui.collectionmenu import CollectionMenu
+from picard.ui.scriptsmenu import ScriptsMenu
 
 
-class BaseAction(QtGui.QAction):
+class BaseAction(QtWidgets.QAction):
     NAME = "Unknown"
     MENU = []
 
     def __init__(self):
-        QtGui.QAction.__init__(self, self.NAME, None)
+        super().__init__(self.NAME, None)
         self.triggered.connect(self.__callback)
 
     def __callback(self):
@@ -84,7 +85,7 @@ def get_match_color(similarity, basecolor):
         c2[2] + (c1[2] - c2[2]) * similarity)
 
 
-class MainPanel(QtGui.QSplitter):
+class MainPanel(QtWidgets.QSplitter):
 
     options = [
         config.Option("persist", "splitter_state", QtCore.QByteArray()),
@@ -97,7 +98,7 @@ class MainPanel(QtGui.QSplitter):
     ]
 
     def __init__(self, window, parent=None):
-        QtGui.QSplitter.__init__(self, parent)
+        super().__init__(parent)
         self.window = window
         self.create_icons()
         self.views = [FileTreeView(window, self), AlbumTreeView(window, self)]
@@ -109,6 +110,8 @@ class MainPanel(QtGui.QSplitter):
         TreeItem.window = window
         TreeItem.base_color = self.palette().base().color()
         TreeItem.text_color = self.palette().text().color()
+        TreeItem.text_color_secondary = self.palette() \
+            .brush(QtGui.QPalette.Disabled, QtGui.QPalette.Text).color()
         TrackItem.track_colors = {
             File.NORMAL: config.setting["color_saved"],
             File.CHANGED: TreeItem.text_color,
@@ -127,12 +130,13 @@ class MainPanel(QtGui.QSplitter):
         for view in self.views:
             view.save_state()
 
+    @restore_method
     def restore_state(self):
         self.restoreState(config.persist["splitter_state"])
 
     def create_icons(self):
-        if hasattr(QtGui.QStyle, 'SP_DirIcon'):
-            ClusterItem.icon_dir = self.style().standardIcon(QtGui.QStyle.SP_DirIcon)
+        if hasattr(QtWidgets.QStyle, 'SP_DirIcon'):
+            ClusterItem.icon_dir = self.style().standardIcon(QtWidgets.QStyle.SP_DirIcon)
         else:
             ClusterItem.icon_dir = icontheme.lookup('folder', icontheme.ICON_SIZE_MENU)
         AlbumItem.icon_cd = icontheme.lookup('media-optical', icontheme.ICON_SIZE_MENU)
@@ -141,7 +145,9 @@ class MainPanel(QtGui.QSplitter):
         AlbumItem.icon_cd_saved_modified = icontheme.lookup('media-optical-saved-modified',
                                                             icontheme.ICON_SIZE_MENU)
         AlbumItem.icon_error = icontheme.lookup('media-optical-error', icontheme.ICON_SIZE_MENU)
-        TrackItem.icon_note = QtGui.QIcon(":/images/note.png")
+        TrackItem.icon_audio = QtGui.QIcon(":/images/track-audio.png")
+        TrackItem.icon_video = QtGui.QIcon(":/images/track-video.png")
+        TrackItem.icon_data = QtGui.QIcon(":/images/track-data.png")
         FileItem.icon_file = QtGui.QIcon(":/images/file.png")
         FileItem.icon_file_pending = QtGui.QIcon(":/images/file-pending.png")
         FileItem.icon_error = icontheme.lookup('dialog-error', icontheme.ICON_SIZE_MENU)
@@ -154,6 +160,14 @@ class MainPanel(QtGui.QSplitter):
             QtGui.QIcon(":/images/match-90.png"),
             QtGui.QIcon(":/images/match-100.png"),
         ]
+        FileItem.match_icons_info = [
+            N_("Bad match"),
+            N_("Poor match"),
+            N_("Ok match"),
+            N_("Good match"),
+            N_("Great match"),
+            N_("Excellent match"),
+        ]
         FileItem.match_pending_icons = [
             QtGui.QIcon(":/images/match-pending-50.png"),
             QtGui.QIcon(":/images/match-pending-60.png"),
@@ -199,7 +213,7 @@ class MainPanel(QtGui.QSplitter):
             self.update_current_view()
 
 
-class BaseTreeView(QtGui.QTreeWidget):
+class BaseTreeView(QtWidgets.QTreeWidget):
 
     options = [
         config.Option("setting", "color_modified", QtGui.QColor(QtGui.QPalette.WindowText)),
@@ -209,7 +223,7 @@ class BaseTreeView(QtGui.QTreeWidget):
     ]
 
     def __init__(self, window, parent=None):
-        QtGui.QTreeWidget.__init__(self, parent)
+        super().__init__(parent)
         self.window = window
         self.panel = parent
 
@@ -220,7 +234,7 @@ class BaseTreeView(QtGui.QTreeWidget):
         self.setAcceptDrops(True)
         self.setDragEnabled(True)
         self.setDropIndicatorShown(True)
-        self.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
+        self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
 
         # enable sorting, but don't actually use it by default
         # XXX it would be nice to be able to go to the 'no sort' mode, but the
@@ -228,10 +242,13 @@ class BaseTreeView(QtGui.QTreeWidget):
         self.header().setSortIndicator(-1, QtCore.Qt.AscendingOrder)
         self.setSortingEnabled(True)
 
-        self.expand_all_action = QtGui.QAction(_("&Expand all"), self)
+        self.expand_all_action = QtWidgets.QAction(_("&Expand all"), self)
         self.expand_all_action.triggered.connect(self.expandAll)
-        self.collapse_all_action = QtGui.QAction(_("&Collapse all"), self)
+        self.collapse_all_action = QtWidgets.QAction(_("&Collapse all"), self)
         self.collapse_all_action.triggered.connect(self.collapseAll)
+        self.select_all_action = QtWidgets.QAction(_("Select &all"), self)
+        self.select_all_action.triggered.connect(self.selectAll)
+        self.select_all_action.setShortcut(QtGui.QKeySequence(_("Ctrl+A")))
         self.doubleClicked.connect(self.activate_item)
 
     def contextMenuEvent(self, event):
@@ -241,7 +258,7 @@ class BaseTreeView(QtGui.QTreeWidget):
         obj = item.obj
         plugin_actions = None
         can_view_info = self.window.view_info_action.isEnabled()
-        menu = QtGui.QMenu(self)
+        menu = QtWidgets.QMenu(self)
 
         if isinstance(obj, Track):
             if can_view_info:
@@ -250,18 +267,23 @@ class BaseTreeView(QtGui.QTreeWidget):
             if obj.num_linked_files == 1:
                 menu.addAction(self.window.play_file_action)
                 menu.addAction(self.window.open_folder_action)
+                menu.addAction(self.window.track_search_action)
                 plugin_actions.extend(_file_actions)
             menu.addAction(self.window.browser_lookup_action)
             menu.addSeparator()
             if isinstance(obj, NonAlbumTrack):
                 menu.addAction(self.window.refresh_action)
         elif isinstance(obj, Cluster):
+            if can_view_info:
+                menu.addAction(self.window.view_info_action)
             menu.addAction(self.window.browser_lookup_action)
             menu.addSeparator()
             menu.addAction(self.window.autotag_action)
             menu.addAction(self.window.analyze_action)
-            if isinstance(obj, UnmatchedFiles):
+            if isinstance(obj, UnclusteredFiles):
                 menu.addAction(self.window.cluster_action)
+            else:
+                menu.addAction(self.window.album_search_action)
             plugin_actions = list(_cluster_actions)
         elif isinstance(obj, ClusterList):
             menu.addAction(self.window.autotag_action)
@@ -276,6 +298,7 @@ class BaseTreeView(QtGui.QTreeWidget):
             menu.addSeparator()
             menu.addAction(self.window.autotag_action)
             menu.addAction(self.window.analyze_action)
+            menu.addAction(self.window.track_search_action)
             plugin_actions = list(_file_actions)
         elif isinstance(obj, Album):
             if can_view_info:
@@ -291,7 +314,7 @@ class BaseTreeView(QtGui.QTreeWidget):
         bottom_separator = False
 
         if isinstance(obj, Album) and not isinstance(obj, NatAlbum) and obj.loaded:
-            releases_menu = QtGui.QMenu(_("&Other versions"), menu)
+            releases_menu = QtWidgets.QMenu(_("&Other versions"), menu)
             menu.addSeparator()
             menu.addMenu(releases_menu)
             loading = releases_menu.addAction(_('Loading...'))
@@ -317,8 +340,8 @@ class BaseTreeView(QtGui.QTreeWidget):
                     for version in versions:
                         priority = {
                             "trackmatch": "0" if version['totaltracks'] == albumtracks else "?",
-                            "countrymatch": "0" if len(preferred_countries) == 0 or preferred_countries & set(version['countries']) else "?",
-                            "formatmatch": "0" if len(preferred_formats) == 0 or preferred_formats & set(version['formats']) else "?",
+                            "countrymatch": "0" if len(preferred_countries) == 0 or preferred_countries & set(version['countries'] or '') else "?",
+                            "formatmatch": "0" if len(preferred_formats) == 0 or preferred_formats & set(version['formats'] or '') else "?",
                         }
                         priorities[version['id']] = "".join(priority[k] for k in matches)
                     versions.sort(key=lambda version: priorities[version['id']] + version['name'])
@@ -348,7 +371,7 @@ class BaseTreeView(QtGui.QTreeWidget):
         if config.setting["enable_ratings"] and \
            len(self.window.selected_objects) == 1 and isinstance(obj, Track):
             menu.addSeparator()
-            action = QtGui.QWidgetAction(menu)
+            action = QtWidgets.QWidgetAction(menu)
             action.setDefaultWidget(RatingWidget(menu, obj))
             menu.addAction(action)
             menu.addSeparator()
@@ -361,16 +384,20 @@ class BaseTreeView(QtGui.QTreeWidget):
                 menu.addSeparator()
             menu.addMenu(CollectionMenu(selected_albums, _("Collections"), menu))
 
-        if plugin_actions:
-            plugin_menu = QtGui.QMenu(_("P&lugins"), menu)
-            plugin_menu.setIcon(self.panel.icon_plugins)
+        scripts = config.setting["list_of_scripts"]
+
+        if plugin_actions or scripts:
             menu.addSeparator()
+
+        if plugin_actions:
+            plugin_menu = QtWidgets.QMenu(_("P&lugins"), menu)
+            plugin_menu.setIcon(self.panel.icon_plugins)
             menu.addMenu(plugin_menu)
 
             plugin_menus = {}
             for action in plugin_actions:
                 action_menu = plugin_menu
-                for index in xrange(1, len(action.MENU) + 1):
+                for index in range(1, len(action.MENU) + 1):
                     key = tuple(action.MENU[:index])
                     if key in plugin_menus:
                         action_menu = plugin_menus[key]
@@ -378,14 +405,21 @@ class BaseTreeView(QtGui.QTreeWidget):
                         action_menu = plugin_menus[key] = action_menu.addMenu(key[-1])
                 action_menu.addAction(action)
 
+        if scripts:
+            scripts_menu = ScriptsMenu(scripts, _("&Run scripts"), menu)
+            scripts_menu.setIcon(self.panel.icon_plugins)
+            menu.addMenu(scripts_menu)
+
         if isinstance(obj, Cluster) or isinstance(obj, ClusterList) or isinstance(obj, Album):
             menu.addSeparator()
             menu.addAction(self.expand_all_action)
             menu.addAction(self.collapse_all_action)
 
+        menu.addAction(self.select_all_action)
         menu.exec_(event.globalPos())
         event.accept()
 
+    @restore_method
     def restore_state(self):
         sizes = config.persist[self.view_sizes.name]
         header = self.header()
@@ -398,7 +432,7 @@ class BaseTreeView(QtGui.QTreeWidget):
 
     def save_state(self):
         cols = range(self.numHeaderSections - 1)
-        sizes = " ".join(str(self.header().sectionSize(i)) for i in cols)
+        sizes = " ".join(string_(self.header().sectionSize(i)) for i in cols)
         config.persist[self.view_sizes.name] = sizes
 
     def supportedDropActions(self):
@@ -421,7 +455,7 @@ class BaseTreeView(QtGui.QTreeWidget):
         if items:
             drag = QtGui.QDrag(self)
             drag.setMimeData(self.mimeData(items))
-            drag.start(supportedActions)
+            drag.exec_(QtCore.Qt.MoveAction)
 
     def mimeData(self, items):
         """Return MIME data for specified items."""
@@ -431,17 +465,11 @@ class BaseTreeView(QtGui.QTreeWidget):
         for item in items:
             obj = item.obj
             if isinstance(obj, Album):
-                album_ids.append(str(obj.id))
-            elif isinstance(obj, Track):
-                files.extend(url(file.filename) for file in obj.linked_files)
-            elif isinstance(obj, File):
-                files.append(url(obj.filename))
-            elif isinstance(obj, Cluster):
-                files.extend(url(file.filename) for file in obj.files)
-            elif isinstance(obj, ClusterList):
-                files.extend(url(file.filename) for cluster in obj for file in cluster.files)
+                album_ids.append(string_(obj.id))
+            elif obj.iterfiles:
+                files.extend([url(f.filename) for f in obj.iterfiles()])
         mimeData = QtCore.QMimeData()
-        mimeData.setData("application/picard.album-list", "\n".join(album_ids))
+        mimeData.setData("application/picard.album-list", "\n".join(album_ids).encode())
         if files:
             mimeData.setUrls(files)
         return mimeData
@@ -451,9 +479,9 @@ class BaseTreeView(QtGui.QTreeWidget):
         files = []
         new_files = []
         for url in urls:
+            log.debug("Dropped the URL: %r", url.toString(QtCore.QUrl.RemoveUserInfo))
             if url.scheme() == "file" or not url.scheme():
-                # Dropping a file from iTunes gives a filename with a NULL terminator
-                filename = os.path.normpath(os.path.realpath(unicode(url.toLocalFile()).rstrip("\0")))
+                filename = os.path.normpath(os.path.realpath(url.toLocalFile().rstrip("\0")))
                 file = BaseTreeView.tagger.files.get(filename)
                 if file:
                     files.append(file)
@@ -462,7 +490,7 @@ class BaseTreeView(QtGui.QTreeWidget):
                 else:
                     new_files.append(filename)
             elif url.scheme() in ("http", "https"):
-                path = unicode(url.path())
+                path = url.path()
                 match = re.search(r"/(release|recording)/([0-9a-z\-]{36})", path)
                 if match:
                     entity = match.group(1)
@@ -477,7 +505,7 @@ class BaseTreeView(QtGui.QTreeWidget):
             BaseTreeView.tagger.add_files(new_files, target=target)
 
     def dropEvent(self, event):
-        return QtGui.QTreeView.dropEvent(self, event)
+        return QtWidgets.QTreeView.dropEvent(self, event)
 
     def dropMimeData(self, parent, index, data, action):
         target = None
@@ -493,25 +521,23 @@ class BaseTreeView(QtGui.QTreeWidget):
         # text/uri-list
         urls = data.urls()
         if urls:
-            if target is None:
-                target = self.tagger.unmatched_files
             self.drop_urls(urls, target)
             handled = True
         # application/picard.album-list
         albums = data.data("application/picard.album-list")
         if albums:
             if isinstance(self, FileTreeView) and target is None:
-                target = self.tagger.unmatched_files
-            albums = [self.tagger.load_album(id) for id in str(albums).split("\n")]
+                target = self.tagger.unclustered_files
+            albums = [self.tagger.load_album(id) for id in string_(albums).split("\n")]
             self.tagger.move_files(self.tagger.get_files_from_objects(albums), target)
             handled = True
         return handled
 
     def activate_item(self, index):
         obj = self.itemFromIndex(index).obj
-        # Double-clicking albums should expand them. The album info can be
+        # Double-clicking albums or clusters should expand them. The album info can be
         # viewed by using the toolbar button.
-        if not isinstance(obj, Album) and obj.can_view_info():
+        if not isinstance(obj, (Album, Cluster)) and obj.can_view_info():
             self.window.view_info()
 
     def add_cluster(self, cluster, parent_item=None):
@@ -525,11 +551,11 @@ class BaseTreeView(QtGui.QTreeWidget):
             cluster_item.add_files(cluster.files)
 
     def moveCursor(self, action, modifiers):
-        if action in (QtGui.QAbstractItemView.MoveUp, QtGui.QAbstractItemView.MoveDown):
+        if action in (QtWidgets.QAbstractItemView.MoveUp, QtWidgets.QAbstractItemView.MoveDown):
             item = self.currentItem()
             if item and not item.isSelected():
                 self.setCurrentItem(item)
-        return QtGui.QTreeWidget.moveCursor(self, action, modifiers)
+        return QtWidgets.QTreeWidget.moveCursor(self, action, modifiers)
 
 
 class FileTreeView(BaseTreeView):
@@ -537,15 +563,15 @@ class FileTreeView(BaseTreeView):
     view_sizes = config.TextOption("persist", "file_view_sizes", "250 40 100")
 
     def __init__(self, window, parent=None):
-        BaseTreeView.__init__(self, window, parent)
+        super().__init__(window, parent)
         self.setAccessibleName(_("file view"))
         self.setAccessibleDescription(_("Contains unmatched files and clusters"))
-        self.unmatched_files = ClusterItem(self.tagger.unmatched_files, False, self)
+        self.unmatched_files = ClusterItem(self.tagger.unclustered_files, False, self)
         self.unmatched_files.update()
-        self.setItemExpanded(self.unmatched_files, True)
+        self.unmatched_files.setExpanded(True)
         self.clusters = ClusterItem(self.tagger.clusters, False, self)
         self.set_clusters_text()
-        self.setItemExpanded(self.clusters, True)
+        self.clusters.setExpanded(True)
         self.tagger.cluster_added.connect(self.add_file_cluster)
         self.tagger.cluster_removed.connect(self.remove_file_cluster)
 
@@ -559,7 +585,7 @@ class FileTreeView(BaseTreeView):
         self.set_clusters_text()
 
     def set_clusters_text(self):
-        self.clusters.setText(0, '%s (%d)' % (_(u"Clusters"), len(self.tagger.clusters)))
+        self.clusters.setText(0, '%s (%d)' % (_("Clusters"), len(self.tagger.clusters)))
 
 
 class AlbumTreeView(BaseTreeView):
@@ -567,7 +593,7 @@ class AlbumTreeView(BaseTreeView):
     view_sizes = config.TextOption("persist", "album_view_sizes", "250 40 100")
 
     def __init__(self, window, parent=None):
-        BaseTreeView.__init__(self, window, parent)
+        super().__init__(window, parent)
         self.setAccessibleName(_("album view"))
         self.setAccessibleDescription(_("Contains albums and matched files"))
         self.tagger.album_added.connect(self.add_album)
@@ -588,12 +614,12 @@ class AlbumTreeView(BaseTreeView):
         self.takeTopLevelItem(self.indexOfTopLevelItem(album.item))
 
 
-class TreeItem(QtGui.QTreeWidgetItem):
+class TreeItem(QtWidgets.QTreeWidgetItem):
 
     __lt__ = lambda self, other: False
 
     def __init__(self, obj, sortable, *args):
-        QtGui.QTreeWidgetItem.__init__(self, *args)
+        super().__init__(*args)
         self.obj = obj
         if obj is not None:
             obj.item = self
@@ -611,7 +637,7 @@ class TreeItem(QtGui.QTreeWidgetItem):
 class ClusterItem(TreeItem):
 
     def __init__(self, *args):
-        TreeItem.__init__(self, *args)
+        super().__init__(*args)
         self.setIcon(0, ClusterItem.icon_dir)
 
     def update(self):
@@ -654,11 +680,11 @@ class AlbumItem(TreeItem):
             oldnum = self.childCount() - 1
             newnum = len(album.tracks)
             if oldnum > newnum:  # remove old items
-                for i in xrange(oldnum - newnum):
+                for i in range(oldnum - newnum):
                     self.takeChild(newnum - 1)
                 oldnum = newnum
             # update existing items
-            for i in xrange(oldnum):
+            for i in range(oldnum):
                 item = self.child(i)
                 track = album.tracks[i]
                 item.obj = track
@@ -666,7 +692,7 @@ class AlbumItem(TreeItem):
                 item.update(update_album=False)
             if newnum > oldnum:  # add new items
                 items = []
-                for i in xrange(newnum - 1, oldnum - 1, -1):  # insertChildren is backwards
+                for i in range(newnum - 1, oldnum - 1, -1):  # insertChildren is backwards
                     item = TrackItem(album.tracks[i], False)
                     item.setHidden(False)  # Workaround to make sure the parent state gets updated
                     items.append(item)
@@ -675,16 +701,21 @@ class AlbumItem(TreeItem):
                     item.update(update_album=False)
         if album.errors:
             self.setIcon(0, AlbumItem.icon_error)
+            self.setToolTip(0, _("Error"))
         elif album.is_complete():
             if album.is_modified():
                 self.setIcon(0, AlbumItem.icon_cd_saved_modified)
+                self.setToolTip(0, _("Album modified and complete"))
             else:
                 self.setIcon(0, AlbumItem.icon_cd_saved)
+                self.setToolTip(0, _("Album unchanged and complete"))
         else:
             if album.is_modified():
                 self.setIcon(0, AlbumItem.icon_cd_modified)
+                self.setToolTip(0, _("Album modified"))
             else:
                 self.setIcon(0, AlbumItem.icon_cd)
+                self.setToolTip(0, _("Album unchanged"))
         for i, column in enumerate(MainPanel.columns):
             self.setText(i, album.column(column[1]))
         if self.isSelected():
@@ -693,7 +724,7 @@ class AlbumItem(TreeItem):
 
 class TrackItem(TreeItem):
 
-    def update(self, update_album=True):
+    def update(self, update_album=True, update_files=True):
         track = self.obj
         if track.num_linked_files == 1:
             file = track.linked_files[0]
@@ -701,30 +732,41 @@ class TrackItem(TreeItem):
             color = TrackItem.track_colors[file.state]
             bgcolor = get_match_color(file.similarity, TreeItem.base_color)
             icon = FileItem.decide_file_icon(file)
+            self.setToolTip(0, _(FileItem.decide_file_icon_info(file)))
             self.takeChildren()
         else:
-            color = TreeItem.text_color
+            self.setToolTip(0, "")
+            if track.ignored_for_completeness():
+                color = TreeItem.text_color_secondary
+            else:
+                color = TreeItem.text_color
             bgcolor = get_match_color(1, TreeItem.base_color)
-            icon = TrackItem.icon_note
-            oldnum = self.childCount()
-            newnum = track.num_linked_files
-            if oldnum > newnum:  # remove old items
-                for i in xrange(oldnum - newnum):
-                    self.takeChild(newnum - 1).obj.item = None
-                oldnum = newnum
-            for i in xrange(oldnum):  # update existing items
-                item = self.child(i)
-                file = track.linked_files[i]
-                item.obj = file
-                file.item = item
-                item.update()
-            if newnum > oldnum:  # add new items
-                items = []
-                for i in xrange(newnum - 1, oldnum - 1, -1):
-                    item = FileItem(track.linked_files[i], False)
-                    item.update()
-                    items.append(item)
-                self.addChildren(items)
+            if track.is_video():
+                icon = TrackItem.icon_video
+            elif track.is_data():
+                icon = TrackItem.icon_data
+            else:
+                icon = TrackItem.icon_audio
+            if update_files:
+                oldnum = self.childCount()
+                newnum = track.num_linked_files
+                if oldnum > newnum:  # remove old items
+                    for i in range(oldnum - newnum):
+                        self.takeChild(newnum - 1).obj.item = None
+                    oldnum = newnum
+                for i in range(oldnum):  # update existing items
+                    item = self.child(i)
+                    file = track.linked_files[i]
+                    item.obj = file
+                    file.item = item
+                    item.update(update_track=False)
+                if newnum > oldnum:  # add new items
+                    items = []
+                    for i in range(newnum - 1, oldnum - 1, -1):
+                        item = FileItem(track.linked_files[i], False)
+                        item.update(update_track=False)
+                        items.append(item)
+                    self.addChildren(items)
             self.setExpanded(True)
         self.setIcon(0, icon)
         for i, column in enumerate(MainPanel.columns):
@@ -739,7 +781,7 @@ class TrackItem(TreeItem):
 
 class FileItem(TreeItem):
 
-    def update(self):
+    def update(self, update_track=True):
         file = self.obj
         self.setIcon(0, FileItem.decide_file_icon(file))
         color = FileItem.file_colors[file.state]
@@ -751,6 +793,10 @@ class FileItem(TreeItem):
         if self.isSelected():
             TreeItem.window.update_selection()
 
+        parent = self.parent()
+        if isinstance(parent, TrackItem) and update_track:
+            parent.update(update_files=False)
+
     @staticmethod
     def decide_file_icon(file):
         if file.state == File.ERROR:
@@ -766,3 +812,16 @@ class FileItem(TreeItem):
             return FileItem.icon_file_pending
         else:
             return FileItem.icon_file
+
+    @staticmethod
+    def decide_file_icon_info(file):
+        # Note error state info is already handled
+        if isinstance(file.parent, Track):
+            if file.state == File.NORMAL:
+                return N_("Track saved")
+            elif file.state == File.PENDING:   # unsure how to use int(file.similarity * 5 + 0.5)
+                return N_("Pending")
+            else:   # returns description of the match ranging from bad to excellent
+                return FileItem.match_icons_info[int(file.similarity * 5 + 0.5)]
+        elif file.state == File.PENDING:
+            return N_("Pending")
