@@ -1,7 +1,5 @@
 # -*- coding: utf-8 -*-
 
-from __future__ import print_function
-import os.path
 import unittest
 from picard import util
 #from picard.util import textencoding
@@ -103,7 +101,7 @@ combinations_to = (
     u"jLqdzdztslslzBDLuebdfmnprrstzthIpUbdfgklmnprsvx"
     u"zadeeiussSSLLllVvYy(C)(R)CECrFr.L.PtsTLRsRx,.x/.ddHhts"
     )
-ascii = u" !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~"
+ascii_chars = u" !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~"
 
 
 class CompatibilityTest(unittest.TestCase):
@@ -113,7 +111,7 @@ class CompatibilityTest(unittest.TestCase):
         self.assertEqual(util.textencoding.unicode_simplify_compatibility(compatibility_from), compatibility_to)
         self.assertEqual(util.textencoding.unicode_simplify_compatibility(punctuation_from), punctuation_from)
         self.assertEqual(util.textencoding.unicode_simplify_compatibility(combinations_from), combinations_from)
-        self.assertEqual(util.textencoding.unicode_simplify_compatibility(ascii), ascii)
+        self.assertEqual(util.textencoding.unicode_simplify_compatibility(ascii_chars), ascii_chars)
 
     def test_incorrect(self):
         pass
@@ -126,7 +124,11 @@ class PunctuationTest(unittest.TestCase):
         self.assertEqual(util.textencoding.unicode_simplify_punctuation(compatibility_from), compatibility_from)
         self.assertEqual(util.textencoding.unicode_simplify_punctuation(punctuation_from), punctuation_to)
         self.assertEqual(util.textencoding.unicode_simplify_punctuation(combinations_from), combinations_from)
-        self.assertEqual(util.textencoding.unicode_simplify_punctuation(ascii), ascii)
+        self.assertEqual(util.textencoding.unicode_simplify_punctuation(ascii_chars), ascii_chars)
+
+    def test_pathsave(self):
+        self.assertEqual(util.textencoding.unicode_simplify_punctuation('\u2215', True), '_')
+        self.assertEqual(util.textencoding.unicode_simplify_punctuation('/\\\u2215', True), '/\\_')
 
     def test_incorrect(self):
         pass
@@ -139,7 +141,11 @@ class CombinationsTest(unittest.TestCase):
         self.assertEqual(util.textencoding.unicode_simplify_combinations(combinations_from), combinations_to)
         self.assertEqual(util.textencoding.unicode_simplify_combinations(compatibility_from), compatibility_from)
         self.assertEqual(util.textencoding.unicode_simplify_combinations(punctuation_from), punctuation_from)
-        self.assertEqual(util.textencoding.unicode_simplify_combinations(ascii), ascii)
+        self.assertEqual(util.textencoding.unicode_simplify_combinations(ascii_chars), ascii_chars)
+
+    def test_pathsave(self):
+        self.assertEqual(util.textencoding.unicode_simplify_combinations('8½', True), '8 1_2')
+        self.assertEqual(util.textencoding.unicode_simplify_combinations('8/\\½', True), '8/\\ 1_2')
 
     def test_incorrect(self):
         pass
@@ -191,6 +197,9 @@ class ReplaceNonAsciiTest(unittest.TestCase):
         self.assertEqual(util.textencoding.replace_non_ascii(u"⒈ ⒉ ⒊"), u"1. 2. 3.") # Digit full stop
         self.assertEqual(util.textencoding.replace_non_ascii(u"１２３"), u"123") # Fullwidth digits
 
+    def test_pathsave(self):
+        self.assertEqual(util.textencoding.replace_non_ascii('\u2044/8½\\', pathsave=True), '_/8 1_2\\')
+
     def test_incorrect(self):
         self.assertNotEqual(util.textencoding.replace_non_ascii(u"Lukáš"), u"Lukáš")
         self.assertNotEqual(util.textencoding.replace_non_ascii(u"Lukáš"), u"Luk____")
